from django import forms
from django.utils.safestring import mark_safe
from wagtail.admin.forms import WagtailAdminPageForm
from wagtail.models.collections import Collection

from shared.jekyll_import import JekyllArticleImporter


class JekyllImportForm(WagtailAdminPageForm):
    do_import = forms.BooleanField(
        initial=False, required=False, label="Provést import z Jekyllu"
    )
    collection = forms.ModelChoiceField(
        queryset=Collection.objects.all(), required=False, label="Kolekce obrázků"
    )
    dry_run = forms.BooleanField(
        initial=True,
        required=False,
        label="Jenom na zkoušku",
        help_text="Žádné články se neuloží, vypíše případné problémy či "
        "již existující články - 'ostrému' importu existující "
        "články nevadí, přeskočí je",
    )
    use_git = forms.BooleanField(
        initial=False,
        required=False,
        label="Použít Git",
        help_text="Umožňuje jednodušší zpracování, ale vyžaduje nainstalovaný Git.",
    )
    jekyll_repo_url = forms.URLField(
        max_length=512,
        required=False,
        help_text=mark_safe(
            "V GitHubu tlačítko Code -> a odkaz z <strong>Download zip</strong>, "
            "např. <em>https://github.com/pirati-web/cb.pirati.cz/archive/refs/heads/gh-pages.zip</em>. "
            "Pokud máte nainstalovaný Git, zvolte <strong>Použít Git</strong> a vložte jednoduše "
            "URL repozitáře, např. <em>https://github.com/pirati-web/cb.pirati.cz</em>."
        ),
    )
    readonly_log = forms.CharField(
        disabled=True,
        label="Log z posledního importu",
        required=False,
        widget=forms.Textarea,
    )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields["readonly_log"].initial = self.instance.last_import_log

    def clean(self):
        cleaned_data = super().clean()

        if not cleaned_data.get("do_import"):
            return cleaned_data

        if cleaned_data.get("do_import") and not self.instance.id:
            self.add_error(
                "do_import", "Import proveďte prosím až po vytvoření stránky"
            )

        if not cleaned_data.get("collection"):
            self.add_error("collection", "Pro import je toto pole povinné")
        if not cleaned_data.get("jekyll_repo_url"):
            self.add_error("jekyll_repo_url", "Pro import je toto pole povinné")

        if cleaned_data.get("use_git"):
            if cleaned_data.get("jekyll_repo_url", "").endswith(".zip"):
                self.add_error(
                    "jekyll_repo_url", "Vložte odkaz pouze na repozitář, ne na zip"
                )
            elif cleaned_data.get("jekyll_repo_url", "").endswith(".git"):
                self.add_error(
                    "jekyll_repo_url",
                    "Vložte odkaz pouze na repozitář, ne na '.git' soubor",
                )
        else:
            if not cleaned_data.get("jekyll_repo_url", "").endswith(".zip"):
                self.add_error("jekyll_repo_url", "Odkaz nesměřuje na soubor '.zip'")

        return cleaned_data

    def handle_import(self):
        from .models import DistrictArticlePage

        JekyllArticleImporter(
            article_parent_page=self.instance,
            collection_id=self.cleaned_data["collection"].id,
            url=self.cleaned_data["jekyll_repo_url"],
            dry_run=self.cleaned_data["dry_run"],
            use_git=self.cleaned_data["use_git"],
            page_model=DistrictArticlePage,
        ).perform_import()

    def save(self, commit=True):
        if self.cleaned_data.get("do_import"):
            self.handle_import()

        return super().save(commit=commit)
