from dateutil.relativedelta import relativedelta
from django.conf import settings
from django.contrib import messages
from django.core.paginator import Paginator
from django.db import models
from django.forms.models import model_to_dict
from django.http import HttpResponseRedirect, JsonResponse
from django.shortcuts import redirect, render
from django.utils import timezone
from modelcluster.contrib.taggit import ClusterTaggableManager
from modelcluster.fields import ParentalKey
from taggit.models import Tag, TaggedItemBase
from wagtail.admin.panels import (
    FieldPanel,
    HelpPanel,
    MultiFieldPanel,
    ObjectList,
    PageChooserPanel,
    TabbedInterface,
)
from wagtail.blocks import PageChooserBlock, RichTextBlock
from wagtail.contrib.routable_page.models import RoutablePageMixin, route
from wagtail.fields import RichTextField, StreamField
from wagtail.models import Page
from wagtail.search import index
from wagtailmetadata.models import MetadataPageMixin

from calendar_utils.models import CalendarMixin
from shared import blocks as shared_blocks
from shared.forms import SubscribeForm
from shared.models import (  # MenuMixin,
    ArticleMixin,
    ArticlesMixin,
    ArticlesPageMixin,
    ExtendedMetadataHomePageMixin,
    ExtendedMetadataPageMixin,
    MainArticlePageMixin,
    MainArticlesPageMixin,
    MainContactPageMixin,
    MainHomePageMixin,
    MainSearchPageMixin,
    MainSimplePageMixin,
    PageInMenuMixin,
    SharedTaggedElectionsArticle,
    SubpageMixin,
)
from shared.utils import make_promote_panels, subscribe_to_newsletter
from tuning import admin_help

from . import blocks


class ElectionsHomePage(MainHomePageMixin):
    # content
    content = StreamField(
        [
            ("carousel", blocks.ElectionsCarouselBlock()),
            ("candidate", blocks.CandidateListBlock()),
            ("program", blocks.ProgramBlock()),
            (
                "news",
                shared_blocks.NewsBlock(
                    template="styleguide2/includes/organisms/articles/elections/articles_section.html"
                ),
            ),
        ],
        verbose_name="Hlavní obsah",
        blank=True,
        use_json_field=True,
    )
    # settings
    gdpr_and_cookies_page = models.ForeignKey(
        # Keep this a reference to the main website's GDPR page. Not ideal, but works (for now).
        "main.MainSimplePage",
        verbose_name="Stránka pro GDPR",
        on_delete=models.PROTECT,
        blank=True,
        null=True,
    )

    subpage_types = [
        "elections.ElectionsArticlesPage",
        "elections.ElectionsCandidatesPage",
        "elections.ElectionsFaqPage",
        "elections.ElectionsSearchPage",
    ]

    ### OTHERS

    class Meta:
        verbose_name = "Volební web"

    @property
    def article_page_model(self):
        return ElectionsArticlePage

    @property
    def articles_page_model(self):
        return ElectionsArticlesPage

    @property
    def search_page_model(self):
        return ElectionsSearchPage

    @property
    def root_page(self):
        return self

    def _first_subpage_of_type(self, page_type) -> Page or None:
        try:
            return self.get_descendants().type(page_type).live().specific()[0]
        except IndexError:
            return None

    @route(r"^sdilene/$", name="shared")
    def shared(self, request):
        return self.setup_article_page_context(request)


class ElectionsArticlesPage(MainArticlesPageMixin):
    parent_page_types = ["elections.ElectionsHomePage"]
    subpage_types = ["elections.ElectionsArticlePage"]


class ElectionsArticleTag(TaggedItemBase):
    content_object = ParentalKey(
        "elections.ElectionsArticlePage",
        on_delete=models.CASCADE,
        related_name="elections_tagged_items",
    )


class ElectionsArticlePage(MainArticlePageMixin):
    author_page = models.ForeignKey(
        "elections.ElectionsCandidatePage",
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
        verbose_name="Stránka autora (kandidáta)",
    )
    tags = ClusterTaggableManager(
        through=ElectionsArticleTag,
        related_name="elections_tagged_articles",
        blank=True,
    )
    shared_tags = ClusterTaggableManager(
        verbose_name="Tagy pro sdílení mezi weby",
        through=SharedTaggedElectionsArticle,
        blank=True,
    )

    parent_page_types = ["elections.ElectionsArticlesPage"]
    subpage_types = []


class ElectionsSearchPage(MainSearchPageMixin):
    parent_page_types = ["elections.ElectionsHomePage"]
    subpage_types = []

    @property
    def searchable_models(self) -> list:
        return [
            ElectionsArticlePage,
            ElectionsCandidatePage,
        ]


class ElectionsCandidatesPage(
    ExtendedMetadataPageMixin,
    SubpageMixin,
    MetadataPageMixin,
    PageInMenuMixin,
    Page,
):
    parent_page_types = ["elections.ElectionsHomePage"]
    subpage_types = ["elections.ElectionsCandidatePage"]

    def serve(self, request, *args, **kwargs):
        return redirect(self.root_page.url)

    class Meta:
        verbose_name = "Kandidáti"


class ElectionsCandidatePage(
    ExtendedMetadataPageMixin,
    SubpageMixin,
    MetadataPageMixin,
    PageInMenuMixin,
    Page,
):
    ## FIELDS

    image = models.ForeignKey(
        "wagtailimages.Image",
        on_delete=models.PROTECT,
        blank=True,
        null=True,
        verbose_name="Obrázek",
        related_name="+",
    )

    position = models.CharField("Pozice/povolání", max_length=64, blank=True, null=True)

    perex = RichTextField()

    email = models.CharField("E-mail", max_length=128, blank=True, null=True)
    phone = models.CharField("Telefonní kontakt", max_length=16, blank=True, null=True)

    social_links = StreamField(
        [
            ("social_links", shared_blocks.SocialLinkBlock()),
        ],
        verbose_name="Odkazy na sociální sítě",
        blank=True,
        use_json_field=True,
    )

    articles_heading = models.CharField(
        "Nadpis sekce s články",
        max_length=32,
        help_text='Např. "Aktuality Mikuláše Peksy".',
    )

    related_candidates = StreamField(
        [
            (
                "candidate",
                PageChooserBlock(
                    page_type="elections.ElectionsCandidatePage",
                    label="Detail kandidáta",
                ),
            )
        ],
        verbose_name="Další kandidáti",
        blank=True,
        use_json_field=True,
    )

    ## PANELS

    content_panels = Page.content_panels + [
        FieldPanel("image"),
        FieldPanel("position"),
        FieldPanel("perex"),
        FieldPanel("email"),
        FieldPanel("phone"),
        FieldPanel("social_links"),
        FieldPanel("articles_heading"),
        FieldPanel("related_candidates"),
    ]

    promote_panels = make_promote_panels()

    ## RELATIONS

    parent_page_types = ["elections.ElectionsCandidatesPage"]
    subpage_types = []

    ## META

    class Meta:
        verbose_name = "Kandidát"

    ## FUNCTIONS

    def get_context(self, request) -> dict:
        context = super().get_context(request)

        context["article_data_list"] = (
            ElectionsArticlePage.objects.filter(author_page=self.id)
            .order_by("-date")
            .live()[:3]
        )

        return context


class ElectionsFaqPage(
    ExtendedMetadataPageMixin,
    SubpageMixin,
    MetadataPageMixin,
    PageInMenuMixin,
    Page,
):
    ## FIELDS

    # content
    content = StreamField(
        [
            ("person_answers", blocks.PersonFaqAnswersBlock()),
        ],
        verbose_name="Obsah",
        blank=True,
        use_json_field=True,
    )

    all_topics_name = models.CharField(
        "Nadpis štítku pro všechna témata",
        max_length=32,
        help_text="např. 'všechna témata'",
    )

    ## PANELS

    content_panels = Page.content_panels + [
        FieldPanel("content"),
        FieldPanel("all_topics_name"),
    ]

    promote_panels = make_promote_panels()

    ## RELATIONS

    parent_page_types = ["elections.ElectionsHomePage"]
    subpage_types = []

    class Meta:
        verbose_name = "Otázky a odpovědi"

    ## OTHERS

    def get_topic_names(self) -> list:
        topic_names = [self.all_topics_name]

        for section in self.content:
            topic_names.append(section.value["topic"])

        return topic_names
